
# Selectpick插件开发流程：
Selectpick是一款轻量级jquery下拉框插件，解决了默认样式外观问题。
实现了数据源动态绑定，样式美化和自定义功能。
通过GULP实现代码压缩和项目构建，可绑定在任何的HTML标签上。
## 开发思路：
jquery插件一般有三种开发方式：

- 通过$.extend()来扩展jQuery
- 通过$.fn 向jQuery添加新的方法
- 通过$.widget()应用jQuery UI的部件工厂方式创建

一般来说，第二种方式比较常见
插件开发，我们一般运用面向对象的思维方式

例如定义一个对象
```javascript
var Haorooms= function(el, opt) {
    this.$element = el,
    this.defaults = {
        'color': 'red',
        'fontSize': '12px',
        'textDecoration':'none'
    },
    this.options = $.extend({}, this.defaults, opt)
}
//定义haorooms的方法
haorooms.prototype = {
    changecss: function() {
        return this.$element.css({
            'color': this.options.color,
            'fontSize': this.options.fontSize,
            'textDecoration': this.options.textDecoration
        });
    }
}
```

## 上述开发方法的问题
上面的开发方法存在一个严重的问题，就是定义了一个全局的Haorooms，这样对于插件的兼容等等各个方面都不好。万一别的地方用到了Haorooms，那么你的代码就悲催了！现在我们把上面的代码包装起来，用一个自调用匿名函数(有时又叫块级作用域或者私有作用域)包裹，就不会出现这个问题了！包括js插件的开发，也是一样的，我们用一个自调用匿名函数把自己写的代码包裹起来，就可以了！包裹方法如下：
```javascript
(function(){

})()
```
用上面的这个包裹起来，就可以了。

但是还有一个问题，当我们研究大牛的代码的时候，前面经常看到有**“；”**，那是为了避免代码合并等不必要的错误。

由于haoroomsblog这个函数后面没有加分号，导致代码出错，为了避免这类情况的发生，通常这么写！
```javascript
；(function(){

    })()
```

说完这些，我们来开始开发一款插件吧～～
## 1.UMD 封装
用这个方式是为了以后插件能够兼容amd,cmd规范，可以支持全局方式加载，以及 AMD 和 CMD 方式加载 
https://github.com/umdjs/umd
```javascript
(function(root, factory) {
        if (typeof define === 'function' && define.amd) {
            define('jqname', ['jquery'], factory);
            } else if (typeof exports === 'object') {
                 module.exports = factory(require('jquery'));
        } else {
            factory(root.jQuery);
        }
    }(this, function($) {
        /* code1 */
        这里写插件源码
    }));

```

## 2.添加自定义的插件	
```javascript
$.fn.Select  = function(settings) {
		var arr = [];
		$(this).each(function() {
			var options = $.extend({
				target: this,
				maxHeight: 200,
				maxWidth: null,
				disabled: false
			}, settings);
			var select = new Select();
			select.init(options);
			arr.push(select);
		});
		return $(arr);
	};
    ```
    
这里我实现的是一个select插件，调用时可以直接$('#sel').Select（{参数列表}）
 此时会对调用jquery的each方法，将默认设置和传递的参数合并为options
 

## 3. 支持链式调用
我们都知道jQuery一个时常优雅的特性是支持链式调用，选择好DOM元素后可以不断地调用其他方法。

要让插件不打破这种链式调用，只需return一下即可。

```javascript
$.fn.Select = function(settings) {
		var arr = [];
        $(this).each(function() {
            var select =  new Select(this,settings);
            select.init();
            arr.push(select);
        });
		return $(arr); ///这里return 为了支持链式调用,遍历所有调用插件的集合。

    };

```
注意调用插件的this是一个数组，需要使用$(this).each遍历数组生成插件。
## 4. 让插件接收参数
一个强劲的插件是可以让使用者随意定制的，这要求我们提供在编写插件时就要考虑得全面些，尽量提供合适的参数。

要做到这一点很方便，只需要使用者在调用的时候传入一个参数即可。同时我们在插件的代码里面接收。另一方面，为了灵活，使用者可以不传递参数，插件里面会给出参数的默认值。

在处理插件参数的接收上，通常使用jQuery的extend方法，上面也提到过，但那是给extend方法传递单个对象的情况下，这个对象会合并到jQuery身上，所以我们就可以在jQuery身上调用新合并对象里包含的方法了。当给extend方法传递一个以上的参数时，它会将所有参数对象合并到第一个里。同时，如果对象中有同名属性时，合并的时候后面的会覆盖前面的。

```javascript
$.fn.myPlugin = function(options) {
    var defaults = {
        'color': 'red',
        'fontSize': '12px'
    };
    var settings = $.extend(defaults, options);
    return this.css({
        'color': settings.color,
        'fontSize': settings.fontSize
    });
}
```
利用这一点，我们可以在插件里定义一个保存插件参数默认值的对象，同时将接收来的参数对象合并到默认对象上，最后就实现了用户指定了值的参数使用指定的值，未指定的参数使用插件默认值。


#### 保护好默认参数

注意到上面代码调用extend时会将defaults的值改变，这样不好，因为它作为插件因有的一些东西应该维持原样，另外就是如果你在后续代码中还要使用这些默认值的话，当你再次访问它时它已经被用户传进来的参数更改了。

 ## 3.添加插件方法
 主要分为以下几个模块
 分为三个子组件，trigger，select和target
 
 - target代表页面的默认元素，页面加载时则隐藏默认样式，在后面添加自定义select元素
 - trigger是select主框，点击出现下拉列表
 - select是下拉列表，我们用ul,li生成，可以自定义样式
 #### 1. init
 初始化组件生成，事件绑定。实际就是调用create和bindevent方法
 
 #### 2. create 
 1. 生成待定义的select主框标签元素
 ```html
 <div class="ui-select-trigger"><span></span><i></i></div>
 ```
 
 2. 生成下拉列表元素
 ```html
 <div id="' + this.id + '" class="ui-select"><ul class="ui-select-content"></ul></div>
 ```
 
 3. format函数
 - 格式化默认下拉列表
 - 生成自定义select下拉列表
 
 #### 3. bindEvent
 初始化后对trigger绑定点击事件，显示下拉列表，再次点击则消失。
 

#### 4. show函数
```javascript
        show: function () {
            this.trigger.addClass('active');
            var _this = this;
            var pos = _this.trigger[0].getBoundingClientRect();
            //设置下拉框的位置
            this.select.css({
                left: pos.left,
                top: _this.trigger.offset().top + pos.bottom - pos.top
            }).css({
                maxHeight: this.settings.maxHeight,
                maxWidth: this.settings.maxWidth || pos.right - pos.left,
                minWidth: this.w
            }).show();
            this.status = true;
        }
```
当点击select框时，会获取trigger的相对窗口偏移位置，同时将select-content设为绝对定位，使其可以一直在trigger的下面



[javascript组件开发方式](http://blog.csdn.net/bingqingsuimeng/article/details/44451481)
 
 [jquery组件开发教程](http://www.cnblogs.com/ajianbeyourself/p/5815689.html)